from visumorph import Image

import numpy as np
from PIL import Image as PImage


def rotate(image, rotation, background=None):
    """Rotate an image with a given rotation.

    This function will rotate a VisuMorph image by a given degree of rotation.
    A background image can be specified for filling any empty spaces which
    are not covered by the image after rotation.

    Parameters
    ----------
    image : visumorph.Image
        The original image to be rotated.
    rotation : float
        The desired rotation angle in degrees.
        +ve means clockwise while -ve means counter-clockwise.
    background : visumorph.Image, optional
        The background image to be filled in where the rotated image is not
        covered. Default is None, which a black background image would be used.

    Returns
    -------
    image
        The VisuMorph Image after rotation.

    Raises
    ------
    TypeError
        If the image is not a valid VisuMorph Image and/or rotation given is
        not a number.

    Examples
    --------
    >>> import visumorph as vm
    >>> img = vm.load_image("test.jpg")
    >>> rotated_45_img = vm.rotate(img, 45.0)
    """

    if not isinstance(image, Image):
        raise TypeError("The image is not a valid VisuMorph Image object")

    if background is not None and not isinstance(background, Image):
        raise TypeError("The background image is not a valid VisuMorph Image object")

    if not type(rotation) in (float, int):
        raise TypeError("The rotation degree must be a float or an integer")

    # minimum size of the pre-rotation image to cover
    min_len = np.ceil(np.hypot(*image.image.shape[0:2])).astype(int)

    min_dim_rgb = (min_len, min_len, 3)

    if background is None:
        bg_arr = np.zeros(min_dim_rgb)
    else:
        # calculate the number of times the bg image to be repeated (tiled)
        num_tiles = np.max(np.ceil(min_len / background.image.shape[0:2])).astype(int)
        # tile it!
        bg_arr = np.tile(background.image, (num_tiles, num_tiles, 1))
        # crop it back to min_dim_rgb
        bg_arr = bg_arr[:min_len, :min_len]

    bg_img = PImage.fromarray(bg_arr.astype(np.uint8), mode="RGB")

    # putting the image at the center of the prepared background image
    x_start = (min_len - image.image.shape[1]) // 2
    y_start = (min_len - image.image.shape[0]) // 2
    bg_img.paste(PImage.fromarray(image.image, mode="RGB"), box=(x_start, y_start))

    rotated = np.array(bg_img.rotate(-rotation))

    # crop image back to its original size
    rotated = rotated[
        y_start : y_start + image.image.shape[0],
        x_start : x_start + image.image.shape[1],
    ]

    return Image(rotated)



from visumorph import Image


def flip(image, v=0):
    """Flip image vertically or horizontally.

    Parameters
    ----------
    image : visumorph.Image
        Image to be flipped.
    v : int, optional
        Flip vertically if v=1, horizontally if v=0. By default, v=0.

    Returns
    -------
    image
        Flipped VisuMorph image.

    Raises
    ------
    TypeError
        If the image is not a valid VisuMorph Image and/or v is not 0 or 1.

    Examples
    --------
    >>> import visumorph
    >>> from visumorph.flip import flip
    >>> img = visumorph.load_image("exampleimage.jpg")
    >>> flip(img, v=1)
    """

    if not isinstance(image, Image):
        raise TypeError("The image is not a valid VisuMorph Image object")

    if v not in (0, 1):
        raise TypeError(
            "Invalid value for 'v'. Use 0 for horizontal flip or 1 for vertical flip."
        )

    flipped = image.image.copy()

    if v == 0:
        flipped = flipped[:, ::-1, :]
    else:
        flipped = flipped[::-1, :, :]

    return Image(flipped)



import numpy as np
from visumorph import Image
from PIL import Image as PImage


def scale(image, scale):
    """Enlarge/Shrink an image with a given scale.

    This function will scale a VisuMorph image by a given scale with 1.0 as
    the original scale. The final transformed image would have a dimension of
    (scale * original_width, scale * original_height), rounded top to the
    nearest integer.

    Parameters
    ----------
    image : visumorph.Image
        The original image to be scaled.
    scale : float or int
        The desired scale. The original scale of the image is 1.0.
        A scale larger than 1.0 will result in a final image having a larger
        dimension, and vice versa. Cannot be equal or smaller than 0.0.

    Returns
    -------
    image
        The VisuMorph Image scaled.

    Raises
    ------
    TypeError
        If the image is not a valid VisuMorph Image and/or scale given is
        not a number.
    ValueError
        If the scale provided is equal 0 or is a negative number.

    Examples
    --------
    >>> import visumorph as vm
    >>> img = vm.load_image("test.jpg")
    >>> scaled_img = vm.scale(img, 1.15)
    """

    if not isinstance(image, Image):
        raise TypeError("The provided image must be a valid VisuMorph Image object.")

    if not type(scale) in (float, int):
        raise TypeError("The scale provided must be a number.")

    if scale <= 0:
        raise ValueError("The scale provided must be a positive, non-zero number.")

    img = PImage.fromarray(image.image)
    new_h, new_w = np.ceil(np.array(image.get_dimensions()[:2]) * scale).astype(int)
    resized = img.resize((new_w, new_h))

    return Image(np.array(resized))



from visumorph import Image as VImage
from PIL import Image as PImage
import numpy as np


def change_hue(image, color="white", delta_hue=1):
    """Change the hue of a VisuMorph Image object by blending it with a layer of specified color.

    Parameters
    ----------
    image : visumorph.Image
        A VisuMorph Image object whose hue is to be changed.
    color : str, optional
        The color used for the blending layer (default is "white").
    delta_hue : float or int, optional
        The degree of blending with the color layer. A value of 0 means no change,
        and 1 means complete replacement with the color layer (default is 1).

    Returns
    -------
    Image : visumorph.Image
        A new VisuMorph Image object with the modified hue.

    Raises
    ------
    TypeError
        If the input is not a valid VisuMorph Image object.
    ValueError
        If 'delta_hue' is not a number.

    Examples
    --------
    >>> import visumorph as vm
    >>> img = vm.load_image("test.jpg")
    >>> hue_changed = vm.change_hue(img)
    """

    if not isinstance(image, VImage):
        raise TypeError("The image is not a valid VisuMorph Image object")

    if not isinstance(delta_hue, (int, float)):
        raise ValueError("delta_hue must be a numeric value")

    # Convert the VisuMorph Image to a PIL Image for processing
    im = PImage.fromarray(image.image, mode="RGB")

    # Create a solid color layer for blending
    layer = PImage.new("RGB", im.size, color)

    # Blend the original image with the color layer
    adjusted_image = PImage.blend(im, layer, delta_hue)

    # Convert the adjusted PIL Image back to a NumPy array
    adjusted_image_np = np.array(adjusted_image)

    # Return a new VisuMorph Image object
    return VImage(adjusted_image_np)
